from genshi.builder import tag

from trac.core import implements,Component
from trac.ticket.api import ITicketActionController
from trac.ticket.default_workflow import ConfigurableTicketWorkflow
from trac.ticket.model import Milestone
from trac.util.translation import _

class MilestoneOperation(Component):
    """Sets milestone for specific status.
    
== Example ==
{{{
[ticket-workflow]
resolve.operations = set_resolution,set_milestone
resolve.milestone = invalid,wontfix,duplicate,worksforme->rejected
}}}

When setting status to `duplicate` the milestone will automaitcally change 
to `rejected`.

'''Note:''' if user has changed milestone manually, this workflow operation 
has ''no effect''!

== Configuration ==
Don't forget to add `MilestoneOperation` to the workflow option 
in `[ticket]` section. If there is no workflow option, the line will look 
like this:
{{{
[ticket]
workflow = ConfigurableTicketWorkflow,MilestoneOperation
}}}
"""
    
    implements(ITicketActionController)

    def get_ticket_actions(self, req, ticket):
        actions_we_handle = []
        if req.authname != 'anonymous' and \
                    'TICKET_MODIFY' in req.perm(ticket.resource):
            controller = ConfigurableTicketWorkflow(self.env)
            actions_we_handle = controller.get_actions_by_operation_for_req(
                req, ticket, 'set_milestone')
        self.log.debug('set_milestone handles actions: %r' % actions_we_handle)
        return actions_we_handle

    def get_all_status(self):
        return []

    def render_ticket_action_control(self, req, ticket, action):
        actions = ConfigurableTicketWorkflow(self.env).actions
        label = actions[action]['name']
        
        res_ms = self.__get_resolution_milestone_dict(ticket, action)
        status = ''
        milestone = None
        for i, key in enumerate(res_ms):
            if i > 0:
                status = "%s, '%s'" % (status, key)
            else:
                status = "'%s'" % key
                milestone = res_ms[key]
        
        hint = _("For status %(status)s the milestone will "
                 "set to '%(milestone)s'.", 
                 status=status, milestone=milestone )
        
        control = tag('')
        return (label, control, hint)
    

    def get_ticket_changes(self, req, ticket, action):
        if req.args and action == 'resolve' \
            and req.args.has_key('action_resolve_resolve_resolution'):
            old_milestone = None
            if ticket._old.has_key('milestone'):
                old_milestone = ticket._old['milestone']
                
            if old_milestone == None or ticket['milestone'] == old_milestone:
                new_status = req.args['action_resolve_resolve_resolution']
                new_milestone = self.__get_new_milestone(ticket, action, 
                                                         new_status)
                milestone = Milestone(self.env, new_milestone)
                self.log.info('changed milestone from %s to %s' % \
                              (old_milestone, new_milestone) )
                
                if milestone.exists:
                    return {'milestone': new_milestone}
        return {}

    def __get_new_milestone(self, ticket, action, new_status):
        """Determines the new status"""
        if new_status:
            res = self.__get_resolution_milestone_dict(ticket, action)
            if res and res.has_key(new_status):
                return res[new_status]
        else:
            return None
    
    def __get_resolution_milestone_dict(self, ticket, action):
        transitions = self.config.get('ticket-workflow',
                                      action + '.milestone').strip()
        transition = [x.strip() for x in transitions.split('->')]
        resoltions = [y.strip() for y in transition[0].split(',')]
        res_milestone = {}
        for res in resoltions:
            res_milestone[res] = transition[1]
        return res_milestone
    
    def apply_action_side_effects(self, req, ticket, action):
        pass
